@@ -376,12 +376,15 @@ lookupOtherName bus o = do
             ai <- lookupUniqueName bus u
             return (u, ai)
 
-        -- … but more than one match means we're screwed.
-        several   -> error $ concat [ "internal error: "
-                                    , show o
-                                    , " in several apps: "
-                                    , show several
-                                    ]
+        -- … but more than one match means we've messed up. This can happen
+        -- with logs generated by dbus-monitor --pcap, which doesn't perform
+        -- an initial dump of all names on the bus.
+        (d:ds)    -> do
+            warn $ concat [ unOtherName o
+                          , " owned by several apps: "
+                          , show (d:ds)
+                          ]
+            return d
 
 -- Finds a TaggedBusName in a map of applications
 lookupApp :: Bus
@@ -661,7 +664,7 @@ processOne bus de = case deEvent de of
 processMessage :: Bus
                -> Detailed Message
                -> Renderer ()
-processMessage bus dm@(Detailed _ m _) = do
+processMessage bus dm@(Detailed _ m _ _) = do
     orly <- shouldShow bus m
     when orly $ case m of
         Signal {}       -> do
