@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 module Bustle.Translation
     (
       initTranslation
@@ -5,13 +6,18 @@ module Bustle.Translation
     )
 where
 
+#ifdef USE_HGETTEXT
 import Text.I18N.GetText
 import System.Locale.SetLocale
 import System.IO.Unsafe
 
 import GetText_bustle
+#endif
 
 initTranslation :: IO ()
+__ :: String -> String
+
+#ifdef USE_HGETTEXT
 initTranslation = do
     setLocale LC_ALL (Just "")
     domain <- getMessageCatalogDomain
@@ -21,5 +27,8 @@ initTranslation = do
     return ()
 
 -- FIXME: I do not like this unsafePerformIO one little bit.
-__ :: String -> String
 __ = unsafePerformIO . getText
+#else
+initTranslation = return ()
+__ = id
+#endif
\ No newline at end of file
