@@ -32,46 +32,6 @@ static gboolean verbose = FALSE;
 static gboolean quiet = FALSE;
 static gboolean version = FALSE;
 
-#if GLIB_CHECK_VERSION (2, 30, 0)
-static void
-let_me_quit (GMainLoop *loop)
-{
-  g_unix_signal_add (SIGINT, (GSourceFunc) g_main_loop_quit, loop);
-
-  if (!quiet)
-    g_printf ("Hit Control-C to stop logging.\n");
-}
-#else
-static gboolean
-stdin_func (
-    GPollableInputStream *g_stdin,
-    GMainLoop *loop)
-{
-  char buf[2];
-
-  if (g_pollable_input_stream_read_nonblocking (g_stdin, buf, 1, NULL, NULL)
-      != -1)
-    {
-      g_main_loop_quit (loop);
-      return FALSE;
-    }
-
-  return TRUE;
-}
-
-static void
-let_me_quit (GMainLoop *loop)
-{
-  GInputStream *g_stdin = g_unix_input_stream_new (0, FALSE);
-  GSource *source = g_pollable_input_stream_create_source (
-      G_POLLABLE_INPUT_STREAM (g_stdin), NULL);
-
-  g_source_set_callback (source, (GSourceFunc) stdin_func, loop, NULL);
-  g_source_attach (source, NULL);
-  g_printf ("Hit Enter to stop logging. (Do not hit Control-C.)\n");
-}
-#endif
-
 static gboolean session_specified = FALSE;
 static gboolean system_specified = FALSE;
 static gchar **filenames = NULL;
@@ -109,6 +69,7 @@ parse_arguments (
   gchar *usage;
   GError *error = NULL;
   gboolean ret;
+  gint exit_status = -1;
 
   context = g_option_context_new ("FILENAME");
   g_option_context_add_main_entries (context, entries, NULL);
@@ -121,7 +82,9 @@ parse_arguments (
     {
       fprintf (stderr, "%s\n", error->message);
       fprintf (stderr, "%s", usage);
-      exit (2);
+
+      exit_status = 2;
+      goto out;
     }
 
   if (version)
@@ -130,13 +93,17 @@ parse_arguments (
       fprintf (stdout, "Copyright 2011 Will Thompson <will@willthompson.co.uk>\n");
       fprintf (stdout, "This is free software; see the source for copying conditions.  There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\n");
       fprintf (stdout, "Written by Will Thompson <will@willthompson.co.uk>\n");
-      exit (0);
+
+      exit_status = 0;
+      goto out;
     }
   else if (session_specified && system_specified)
     {
       fprintf (stderr, "You may only specify one of --session and --system\n");
       fprintf (stderr, "%s", usage);
-      exit (2);
+
+      exit_status = 2;
+      goto out;
     }
   else if (system_specified)
     {
@@ -153,10 +120,21 @@ parse_arguments (
     {
       fprintf (stderr, "You must specify exactly one output filename\n");
       fprintf (stderr, "%s", usage);
-      exit (2);
+
+      exit_status = 2;
+      goto out;
     }
 
-  *filename = filenames[0];
+  *filename = g_strdup (filenames[0]);
+
+out:
+  g_free (usage);
+  g_strfreev (filenames);
+  g_option_context_free (context);
+  g_clear_error (&error);
+
+  if (exit_status > -1)
+    exit (exit_status);
 }
 
 static void
@@ -189,13 +167,13 @@ main (
   GError *error = NULL;
   BustlePcapMonitor *pcap;
 
-  g_type_init ();
   parse_arguments (&argc, &argv, &bus_type, &filename);
 
   pcap = bustle_pcap_monitor_new (bus_type, filename, &error);
   if (pcap == NULL)
     {
       fprintf (stderr, "%s", error->message);
+      g_clear_error (&error);
       exit (1);
     }
 
@@ -208,12 +186,17 @@ main (
   if (!quiet)
     g_printf ("Logging D-Bus traffic to '%s'...\n", filename);
 
-  let_me_quit (loop);
+  g_unix_signal_add (SIGINT, (GSourceFunc) g_main_loop_quit, loop);
+
+  if (!quiet)
+    g_printf ("Hit Control-C to stop logging.\n");
+
   g_main_loop_run (loop);
   g_main_loop_unref (loop);
 
   bustle_pcap_monitor_stop (pcap);
   g_object_unref (pcap);
+  g_free (filename);
 
   return 0;
 }
